@@ -28,6 +28,7 @@
 #include "imageresult.h"
 #include "epsresult.h"
 #include "syntaxhelpobject.h"
+#include "completionobject.h"
 #include "defaultvariablemodel.h"
 
 #include <config-cantorlib.h>
@@ -75,7 +76,7 @@ void TestMaxima::testPlot()
     QVERIFY( e!=nullptr );
     QVERIFY( e->result()!=nullptr );
 
-    if(e->result()->type()!= Cantor::EpsResult::Type)
+    if(!e->result())
     {
         waitForSignal(e, SIGNAL(gotResult()));
     }
@@ -236,10 +237,22 @@ void TestMaxima::testSyntaxHelp()
     help->fetchSyntaxHelp();
     waitForSignal(help, SIGNAL(done()));
 
-    qWarning()<<help->toHtml();
     QVERIFY(help->toHtml().contains(QLatin1String("simplify_sum")));
 }
 
+void TestMaxima::testCompletion()
+{
+    Cantor::CompletionObject* help = session()->completionFor(QLatin1String("ask"), 3);
+    waitForSignal(help, SIGNAL(fetchingDone()));
+
+    // Checks all completions for this request
+    // This correct for Maxima 5.41.0
+    const QStringList& completions = help->completions();
+    QVERIFY(completions.contains(QLatin1String("asksign")));
+    QVERIFY(completions.contains(QLatin1String("askinteger")));
+    QVERIFY(completions.contains(QLatin1String("askexp")));
+}
+
 void TestMaxima::testHelpRequest()
 {
     //execute "??print"
@@ -294,5 +307,32 @@ void TestMaxima::testVariableModel()
     QCOMPARE(value2.toString(),QLatin1String("[1,2,3]"));
 }
 
+void TestMaxima::testLoginLogout()
+{
+    // Logout from session twice and all must works fine
+    session()->logout();
+    session()->logout();
+
+    // Login in session twice and all must works fine
+    session()->login();
+    session()->login();
+}
+
+void TestMaxima::testRestartWhileRunning()
+{
+    Cantor::Expression* e1=session()->evaluateExpression(QLatin1String(":lisp (sleep 5)"));
+
+    session()->logout();
+    QCOMPARE(e1->status(), Cantor::Expression::Interrupted);
+    session()->login();
+
+    Cantor::Expression* e2=evalExp( QLatin1String("2+2") );
+
+    QVERIFY(e2 != nullptr);
+    QVERIFY(e2->result() != nullptr);
+
+    QCOMPARE(cleanOutput(e2->result()->data().toString() ), QLatin1String("4"));
+}
+
 QTEST_MAIN( TestMaxima )
 
