@@ -25,6 +25,7 @@
 #include "latexrenderer.h"
 
 #include <QGraphicsLinearLayout>
+#include <QScopedPointer>
 
 #include <QDebug>
 #include <KLocalizedString>
@@ -115,30 +116,26 @@ QDomElement TextEntry::toXml(QDomDocument& doc, KZip* archive)
 {
     Q_UNUSED(archive);
 
-    bool needsEval=false;
+    QScopedPointer<QTextDocument> document(m_textItem->document()->clone());
+
     //make sure that the latex code is shown instead of the rendered formulas
-    QTextCursor cursor = m_textItem->document()->find(QString(QChar::ObjectReplacementCharacter));
+    QTextCursor cursor = document->find(QString(QChar::ObjectReplacementCharacter));
     while(!cursor.isNull())
     {
         QTextCharFormat format = cursor.charFormat();
         if (format.hasProperty(EpsRenderer::CantorFormula))
-        {
             showLatexCode(cursor);
-            needsEval=true;
-        }
 
-        cursor = m_textItem->document()->find(QString(QChar::ObjectReplacementCharacter), cursor);
+        cursor = document->find(QString(QChar::ObjectReplacementCharacter), cursor);
     }
 
-    const QString& html = m_textItem->toHtml();
+    const QString& html = document->toHtml();
     qDebug() << html;
     QDomElement el = doc.createElement(QLatin1String("Text"));
     QDomDocument myDoc = QDomDocument();
     myDoc.setContent(html);
     el.appendChild(myDoc.documentElement().firstChildElement(QLatin1String("body")));
 
-    if (needsEval)
-        evaluate();
     return el;
 }
 
@@ -226,12 +223,12 @@ void TextEntry::updateEntry()
     QTextCursor cursor = m_textItem->document()->find(QString(QChar::ObjectReplacementCharacter));
     while(!cursor.isNull())
     {
-        QTextCharFormat format = cursor.charFormat();
+        QTextImageFormat format=cursor.charFormat().toImageFormat();
         if (format.hasProperty(EpsRenderer::CantorFormula))
         {
             qDebug() << "found a formula... rendering the eps...";
-            QUrl url = format.property(EpsRenderer::ImagePath).toUrl();
-            QSizeF s = worksheet()->epsRenderer()->renderToResource(m_textItem->document(), url);
+            const QUrl& url=QUrl::fromLocalFile(format.property(EpsRenderer::ImagePath).toString());
+            QSizeF s = worksheet()->epsRenderer()->renderToResource(m_textItem->document(), url, QUrl(format.name()));
             qDebug() << "rendering successful? " << s.isValid();
 
             //cursor.deletePreviousChar();
